//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis                         
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-                                
//*  http://apps.jcns.fz-juelich.de/frida                                   
//**************************************************************************************************

//! \file  special.cpp
//! \brief NSpecial: this and that, written ad hoc.


#include <iostream>
#include <fftw3.h>
#include <string>

#include <readplus/ask.hpp>

#include "defs.hpp"
#include "olf.hpp"
#include "mem.hpp"
#include "slice.hpp"
#include "special.hpp"

namespace NSpecial {
    void q_eval( double *par, int m_dat, double *fvec, void *data, int *info);
    void lm_print(int n_par, double *par, int m_dat, double *fvec,
                  void *data, int iflag, int iter, int nfev);
    double q_kernel( double t, double *p );
}

void NSpecial::FourierCosine() 
{
    fftw_plan plan;
    int n;
    
    NOlm::IterateD fiter;
    POld fin;
    while( (fin=fiter()) ) {
        POld fout( fin->new_old() );
        fout->lDoc.push_back("Fourier Cosine");

        cerr << "WARNING: coordinates not changed\n";
        cerr << "WARNING: x not changed\n";
        cerr << "WARNING: dy not calculated\n";
        // *** set coordinates ***
//                olf.xco = CCoord("z", "m");
//                olf.yco = CCoord("B_dim", "B-unit");

        for ( int j=0; j<fin->nJ(); j++ ) {
            PSpec sin = fin->VS(j);
            PSpec sout( new CSpec( *sin ) );
            
            n = sin->size();
                        
            plan = fftw_plan_r2r_1d(
                n, &(sin->y[0]), &(sout->y[0]),
                FFTW_REDFT00,  FFTW_ESTIMATE);
            fftw_execute( plan );

            sout->dy.clear();
            for ( int i=0; i<n; ++i ) {
                sout->y[i] /= sqrt(2.*n);
            }

            fout->V.push_back(sout);
        }
        NOlm::mem_store( fout );
    }
}

void NSpecial::Test()
{
    // diagonalize finite chain force matrix

    POld fout( new COld );
    
    int N = iask("Order N");
    if ( N<2 )
         return;
         
    fout->name = "EV_" +S(N);
    fout->xco = CCoord("n", "");
    fout->yco = CCoord("v_n", "");
    fout->ZCo.push_back( CCoord("p", "") );
    fout->ZCo.push_back( CCoord("nor", "") );

    for ( int p=0; p<N; ++p ){ 
        PSpec s( new CSpec );

        s->z.push_back( p );

        s->resize(N,false);
        for( int n=0; n<N; ++n )
            s->x[n] = n;

        /// eigenvalue:
        double lambda = -2 + 2*cos(p*M_PI/N);

        // recursive calculation of eigenvector:
        s->y[0] = 1;
        s->y[1] = 1+lambda;
        for( int n=2; n<N; ++n )
            s->y[n] = -s->y[n-2] + (2+lambda)*s->y[n-1];

        // normalize:
        double nor = 0;
        for( int n=0; n<N; ++n )
            nor += s->y[n] * s->y[n];
        printf( "p=%i => check sum = %g, norm = %g\n",
                p, s->y[N-2]-(1+lambda)*s->y[N-1], nor );
        s->z.push_back( nor );
        for( int n=0; n<N; ++n )
            s->y[n] /= sqrt(nor);

        fout->V.push_back(s);
    }
    NOlm::mem_store( fout );
}